"""
Global constants and definitions
"""
#********************************************************************
#      File:    core.py
#      Author:  Sam Strachan / Huxley Barbee
#
#      Description:
#       This file contains core definitions
#
#      Copyright (c) 2017 by Cisco Systems, Inc.
#
#       ALL RIGHTS RESERVED. THESE SOURCE FILES ARE THE SOLE PROPERTY
#       OF CISCO SYSTEMS, Inc. AND CONTAIN CONFIDENTIAL  AND PROPRIETARY
#       INFORMATION.  REPRODUCTION OR DUPLICATION BY ANY MEANS OF ANY
#       PORTION OF THIS SOFTWARE WITHOUT PRIOR WRITTEN CONSENT OF
#       CISCO SYSTEMS, Inc. IS STRICTLY PROHIBITED.
#
#*********************************************************************/

# Disable "too many lines"
#pylint: disable=C0302

STATE_ERROR = -1
STATE_STOPPED = 0
STATE_RUNNING = 1
STATE_STARTING = 2
STATE_STOPPING = 3

STATE_STRING = {
    STATE_ERROR: 'Error',
    STATE_STOPPED: 'Stopped',
    STATE_RUNNING: 'Running',
    STATE_STARTING: 'Starting',
    STATE_STOPPING: 'Stopping'
}

TIME_BLINK = 0.005
TIME_PAUSE = 0.5

# TYPES
TYPE_BYTE = 'B'
TYPE_UINT16 = 'H'
TYPE_UINT32 = 'L'
TYPE_UINT64 = 'Q'
TYPE_UINT128 = 'BBBBBBBBBBBBBBBB'
TYPE_UINT160 = 'BBBBBBBBBBBBBBBBBBBB'
TYPE_UINT256 = 'BBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBB'
TYPE_VARIABLE = 'V'
TYPE_UUID = 'uuid'
TYPE_IPV4 = 'ipv4'
TYPE_IPV6 = 'ipv6'
TYPE_MAC = 'mac'

# TIMESTAMPS
TIMESTAMP_GENESIS = 0
TIMESTAMP_NOW = 0xffffffff

# RECORD TYPES
RECORD_PACKET = 2
RECORD_PRIORITY = 4
RECORD_INTRUSION_IMPACT_ALERT = 9
RECORD_RNA_NEW_HOST = 10
RECORD_RNA_NEW_TCP_SERVICE = 11
RECORD_RNA_NEW_UDP_SERVICE = 12
RECORD_RNA_NEW_NET_PROTOCOL = 13
RECORD_RNA_NEW_XPORT_PROTOCOL = 14
RECORD_RNA_NEW_CLIENT_APP = 15
RECORD_RNA_CHANGE_TCP_SERVICE_INFO = 16
RECORD_RNA_CHANGE_UDP_SERVICE_INFO = 17
RECORD_RNA_CHANGE_OS = 18
RECORD_RNA_CHANGE_HOST_TIMEOUT = 19
RECORD_RNA_CHANGE_HOST_REMOVE = 20
RECORD_RNA_CHANGE_HOST_ANR_DELETE = 21
RECORD_RNA_CHANGE_HOPS = 22
RECORD_RNA_CHANGE_TCP_PORT_CLOSED = 23
RECORD_RNA_CHANGE_UDP_PORT_CLOSED = 24
RECORD_RNA_CHANGE_TCP_PORT_TIMEOUT = 25
RECORD_RNA_CHANGE_UDP_PORT_TIMEOUT = 26
RECORD_RNA_CHANGE_MAC_INFO = 27
RECORD_RNA_CHANGE_MAC_ADD = 28
RECORD_RNA_CHANGE_HOST_IP = 29
RECORD_RNA_CHANGE_HOST_TYPE = 31
RECORD_RNA_CHANGE_VLAN_TAG = 34
RECORD_RNA_CHANGE_CLIENT_APP_TIMEOUT = 35
RECORD_RNA_CHANGE_NETBIOS_NAME = 42
RECORD_RNA_CHANGE_HOST_DROPPED = 44
RECORD_RNA_CHANGE_BANNER_UPDATE = 45
RECORD_RNA_USER_ADD_ATTRIBUTE = 46
RECORD_RNA_USER_UPDATE_ATTRIBUTE = 47
RECORD_RNA_USER_DELETE_ATTRIBUTE = 48
RECORD_RNA_CHANGE_TCP_SERVICE_CONFIDENCE = 51
RECORD_RNA_CHANGE_UDP_SERVICE_CONFIDENCE = 52
RECORD_RNA_CHANGE_OS_CONFIDENCE = 53
METADATA_RNA_FINGERPRINT = 54
METADATA_RNA_CLIENT_APPLICATION = 55
METADATA_RNA_VULNERABILITY = 57
METADATA_RNA_CRITICALITY = 58
METADATA_RNA_NETWORK_PROTOCOL = 59
METADATA_RNA_ATTRIBUTE = 60
METADATA_RNA_SCAN_TYPE = 61
RECORD_USER = 62
METADATA_RNA_SERVICE = 63
METADATA_RULE_MESSAGE = 66
METADATA_CLASSIFICATION = 67
METADATA_CORRELATION_POLICY = 69
METADATA_CORRELATION_RULE = 70
RECORD_RNA_CONNECTION_STATISTICS = 71
RECORD_RNA_CONNECTION_CHUNK = 73
RECORD_RNA_USER_SET_OS = 74
RECORD_RNA_USER_SET_SERVICE = 75
RECORD_RNA_USER_DELETE_PROTOCOL = 76
RECORD_RNA_USER_DELETE_CLIENT_APP = 77
RECORD_RNA_USER_DELETE_ADDRESS = 78
RECORD_RNA_USER_DELETE_SERVICE = 79
RECORD_RNA_USER_VULNERABILITIES_VALID = 80
RECORD_RNA_USER_VULNERABILITIES_INVALID = 81
RECORD_RNA_USER_SET_CRITICALITY = 82
RECORD_RNA_USER_SET_ATTRIBUTE_VALUE = 83
RECORD_RNA_USER_DELETE_ATTRIBUTE_VALUE = 84
RECORD_RNA_USER_ADD_HOST = 85
RECORD_RNA_USER_ADD_SERVICE = 86
RECORD_RNA_USER_ADD_CLIENT_APP = 87
RECORD_RNA_USER_ADD_PROTOCOL = 88
RECORD_RNA_USER_ADD_SCAN_RESULT = 89
METADATA_RNA_SOURCE_TYPE = 90
METADATA_RNA_SOURCE_APP = 91
RUA_EVENT_CHANGE_USER_DROPPED = 92
RUA_EVENT_CHANGE_USER_REMOVE = 93
RUA_EVENT_NEW_USER = 94
RUA_EVENT_CHANGE_USER_LOGIN = 95
METADATA_RNA_SOURCE_DETECTOR = 96
RECORD_RUA_USER = 98
RECORD_RNA_NEW_OS = 101
RECORD_RNA_CHANGE_IDENTITY_CONFLICT = 102
RECORD_RNA_CHANGE_IDENTITY_TIMEOUT = 103
RECORD_THIRD_PARTY_SCAN_VULNERABILITY = 106
RECORD_RNA_CHANGE_CLIENT_APP_UPDATE = 107
RECORD_RNA_WEB_APPLICATION_PAYLOAD = 109
RECORD_INTRUSION_EXTRA_DATA = 110
METADATA_INTRUSION_EXTRA_DATA = 111
RECORD_CORRELATION_EVENT = 112
METADATA_SECURITY_ZONE_NAME = 115
METADATA_INTERFACE_NAME = 116
METADATA_ACCESS_CONTROL_POLICY_NAME = 117
METADATA_INTRUSION_POLICY_NAME = 118
METADATA_ACCESS_CONTROL_RULE_ID = 119
METADATA_ACCESS_CONTROL_RULE_ACTION = 120
METADATA_URL_CATEGORY = 121
METADATA_URL_REPUTATION = 122
METADATA_SENSOR = 123
METADATA_ACCESS_CONTROL_POLICY_RULE_REASON = 124
RECORD_MALWARE_EVENT = 125
METADATA_FIREAMP_CLOUD_NAME = 127
METADATA_FIREAMP_EVENT_TYPE = 128
METADATA_FIREAMP_EVENT_SUBTYPE = 129
METADATA_FIREAMP_DETECTOR_TYPE = 130
METADATA_FIREAMP_FILE_TYPE = 131
METADATA_SECURITY_CONTEXT_NAME = 132
RECORD_RULE_DOCUMENTATION_DATA = 140
METADATA_ACCESS_CONTROL_POLICY = 145
METADATA_PREFILTER_POLICY = 146
METADATA_TUNNEL_OR_PREFILTER_RULE = 147
RECORD_RNA_IOC_SET = 160
METADATA_IOC_NAME = 161
METADATA_ICMP_TYPE = 260
METADATA_ICMP_CODE = 270
METADATA_SECURITY_INTELLIGENCE_CATEGORY_DISCOVERY = 280
METADATA_SECURITY_INTELLIGENCE_SRCDEST = 281
METADATA_SECURITY_INTELLIGENCE_CATEGORY_GENERAL = 282
METADATA_REALM = 300
RECORD_ENDPOINT_PROFILE_DATA = 301
METADATA_SECURITY_GROUP = 302
METADATA_DNS_RECORD = 320
METADATA_DNS_RESPONSE = 321
METADATA_SINKHOLE = 322
METADATA_NETMAP_DOMAIN = 350
RECORD_INTRUSION_EVENT = 400
RECORD_FILELOG_EVENT = 500
RECORD_FILELOG_MALWARE_EVENT = 502
METADATA_FILELOG_FILE_TYPE = 510
METADATA_FILELOG_SHA = 511
METADATA_FILELOG_STORAGE = 515
METADATA_FILELOG_SANDBOX = 516
METADATA_FILELOG_SPERO = 517
METADATA_FILELOG_ARCHIVE = 518
METADATA_FILELOG_STATIC_ANALYSIS = 519
METADATA_GEOLOCATION = 520
METADATA_FILE_POLICY_NAME = 530
METADATA_SSL_POLICY = 600
METADATA_SSL_RULE_ID = 601
METADATA_SSL_CIPHER_SUITE = 602
METADATA_SSL_VERSION = 604
METADATA_SSL_SERVER_CERTIFICATE_STATUS = 605
METADATA_SSL_ACTUAL_ACTION = 606
METADATA_SSL_EXPECTED_ACTION = 607
METADATA_SSL_FLOW_STATUS = 608
METADATA_SSL_URL_CATEGORY = 613
METADATA_SSL_CERTIFICATE_DETAILS_DATA = 614
METADATA_RECORD_NETWORK_ANALYSIS_POLICY = 700

# DEPRECATED RECORD TYPES
DEPRECATED_EVENT = 1
DEPRECATED_CLASSIFICATION = 3
DEPRECATED_RULE = 5
DEPRECATED_RNA = 6
DEPRECATED_EVENT2 = 7
DEPRECATED_POLICY_EVENT = 8
DEPRECATED_RNA_CHANGE_HOST_UPDATE = 30
DEPRECATED_RNA_CHANGE_VULN_MAP = 32
DEPRECATED_RNA_CHANGE_FLOW_STATS = 33
DEPRECATED_POLICY_EVENT_V2 = 36
DEPRECATED_RNA_USER_VULN_VALID = 37
DEPRECATED_RNA_USER_VULN_INVALID = 38
DEPRECATED_RNA_USER_DELETE_ADDR = 39
DEPRECATED_RNA_USER_DELETE_SERVICE = 40
DEPRECATED_RNA_USER_SET_CRIICALITY = 41
DEPRECATED_RNA_USER_SET_ATTRIBUTE_VALUE = 49
DEPRECATED_RNA_USER_DELETE_ATTRIBUTE_VALUE = 50
DEPRECATED_METADATA_RNA_CLIENT_APPLICATION_TYPE = 56
DEPRECATED_DETECTION_ENGINE = 64
DEPRECATED_POLICY_EVENT_V3 = 65
DEPRECATED_DETECTION_ENGINE_V2 = 68
DEPRECATED_IDS_EVENT_IPV6 = 72
DEPRECATED_POLICY_EVENT_V5 = 97
DEPRECATED_IDS_EVENT_MPLS = 99
DEPRECATED_IDS_EVENT_IPV6_MPLS = 100
DEPRECATED_IDS_EVENT_VLAN = 104
DEPRECATED_IDS_EVENT_IPV6_VLAN = 105
DEPRECATED_RNA_PAYLOAD_TYPE = 108
DEPRECATED_RUA_EVENT_FAILED_USER_LOGIN = 114
DEPRECATED_RNA_USER_IOC_DELETE = 162
DEPRECATED_RNA_USER_IOC_ENABLE = 163
DEPRECATED_RNA_USER_IOC_DISABLE = 164
DEPRECATED_IDS_EVENT_NG = 207
DEPRECATED_IDS_EVENT_IPV6_NG = 208


# RNA event types.
RNA_TYPE_NEW        = 1000
RNA_TYPE_CHANGE     = 1001
RNA_TYPE_USER_INPUT = 1002
RNA_TYPE_FLOW       = 1003
RUA_TYPE            = 1004
IOC_TYPE            = 1008

# New subtypes (1000)
RNA_NEW_HOST               = 1
RNA_NEW_TCP_SERVICE        = 2
RNA_NEW_NET_PROTO          = 3
RNA_NEW_TRANSPORT_PROTO    = 4
RNA_NEW_IP_TRAFFIC         = 5
RNA_NEW_UDP_SERVICE        = 6
RNA_NEW_CLIENT_APP         = 7
RNA_NEW_OS_FINGERPRINT     = 8

# Change subtypes (1001)
RNA_IP_CHANGED             = 1
RNA_OS_UPDATE              = 2
RNA_IP_REUSED              = 3
RNA_VULN_CHANGED           = 4
RNA_HOPS_CHANGED           = 5
RNA_TCP_SERVICE_UPDATE     = 6
RNA_HOST_TIMEOUT           = 7
RNA_TCP_PORT_CLOSE         = 8
RNA_UDP_PORT_CLOSE         = 9
RNA_UDP_SERVICE_UPDATE     = 10
RNA_TCP_PORT_TIMEOUT       = 11
RNA_UDP_PORT_TIMEOUT       = 12
RNA_MAC_CHANGE             = 13
RNA_ADDITIONAL_MAC         = 14
RNA_HOST_LAST_SEEN         = 15
RNA_HOST_ROUTER_BRIDGE     = 16
RNA_FLOW_STATISTICS        = 17
RNA_VLAN_TAG_UPDATE        = 18
RNA_HOST_DELETED_LIMIT     = 19
RNA_CLIENT_APP_TIMEOUT     = 20
RNA_NETBIOS_NAME           = 21
RNA_NETBIOS_DOMAIN         = 22
RNA_HT_DROPPED             = 23
RNA_BANNER_UPDATE          = 24
RNA_TCP_SERVICE_CONFIDENCE = 25
RNA_UDP_SERVICE_CONFIDENCE = 26
RNA_OS_CONFIDENCE          = 27
RNA_DHCP_INFO              = 28
RNA_IDENTITY_CONFLICT      = 29
RNA_IDENTITY_TIMEOUT       = 30
RNA_SECONDARY_UPDATE       = 31
RNA_CLIENT_APP_UPDATE      = 32

# User Input subtypes (1002)
RNA_SET_VALID_VULN         = 1
RNA_SET_INVALID_VULN       = 2
RNA_DELETE_ADDRESS         = 3
RNA_DELETE_SERVICE         = 4
RNA_SET_HOST_CRITICALITY   = 5
RNA_ADD_HOST_ATTRIBUTE     = 6
RNA_UPDATE_HOST_ATTRIBUTE  = 7
RNA_DELETE_HOST_ATTRIBUTE  = 8
RNA_SET_HOST_ATTRIBUTE     = 9
RNA_CLEAR_HOST_ATTRIBUTE   = 10
RNA_ADD_SCAN_RESULTS       = 11
RNA_USER_VULN_QUALIFY      = 12
RNA_USER_POLICY_CONTROL    = 13
RNA_DELETE_PROTOCOL        = 14
RNA_DELETE_CLIENT_APP      = 15
RNA_SET_OS                 = 16
RNA_ACCOUNT_SEEN           = 17
RNA_ACCOUNT_UPDATE         = 18
RNA_SET_SERVICE            = 19
RNA_DELETE_ADDR_V2         = 20
RNA_DELETE_SERVICE_V2      = 21
RNA_VULN_VALID_V2          = 22
RNA_VULN_INVALID_V2        = 23
RNA_CRITICALITY_V2         = 24
RNA_SET_ATTRIBUTE_VALUE_V2 = 25
RNA_DELETE_ATTRIBUTE_VALUE_V2 = 26
RNA_ADD_HOST               = 27
RNA_ADD_SERVICE            = 28
RNA_ADD_CLIENT_APP         = 29
RNA_ADD_PROTOCOL           = 30
RNA_RELOAD_APP_FPS         = 31
RNA_ACCOUNT_DELETE         = 32
RNA_IOC_DELETE             = 45
RNA_IOC_INVALUD            = 46

# Flow subtypes (1003)
RNA_FLOW_STATS     = 1
RNA_FLOW_CHUNK     = 2

# RUA subtypes (1004)
RUA_USER_INFO      = 1
RUA_LOGIN_INFO     = 2
RUA_REMOVE_USER    = 3
RUA_DROP_USER      = 4

# IOC subtypes (1008)
IOC_SET            = 1

RNA_TYPE_NAMES = {
    RNA_TYPE_NEW : {
        RNA_NEW_HOST                   : 'New Host',
        RNA_NEW_TCP_SERVICE            : 'New TCP Service',
        RNA_NEW_NET_PROTO              : 'New Network Protocol',
        RNA_NEW_TRANSPORT_PROTO        : 'New Transport Protocol',
        RNA_NEW_IP_TRAFFIC             : 'New IP to IP traffic',
        RNA_NEW_UDP_SERVICE            : 'New UDP Service',
        RNA_NEW_CLIENT_APP             : 'New Client Applicastion',
        RNA_NEW_OS_FINGERPRINT         : 'New OS Fingerprint',
    },
    RNA_TYPE_CHANGE : {
        RNA_IP_CHANGED                 : 'Host IP Address Changed',
        RNA_OS_UPDATE                  : 'OS Information Update',
        RNA_IP_REUSED                  : 'Host IP Address Reused',
        RNA_VULN_CHANGED               : 'Vulnerability Changed',
        RNA_HOPS_CHANGED               : 'Hops Change',
        RNA_TCP_SERVICE_UPDATE         : 'TCP Service Information Update',
        RNA_HOST_TIMEOUT               : 'Host Timeout',
        RNA_TCP_PORT_CLOSE             : 'TCP Port Closed',
        RNA_UDP_PORT_CLOSE             : 'UDP Port Closed',
        RNA_UDP_SERVICE_UPDATE         : 'UDP Service Information Update',
        RNA_TCP_PORT_TIMEOUT           : 'TCP Port Timeout',
        RNA_UDP_PORT_TIMEOUT           : 'UDP Port Timeout',
        RNA_MAC_CHANGE                 : 'MAC Information Change',
        RNA_ADDITIONAL_MAC             : 'Additional MAC Detected',
        RNA_HOST_LAST_SEEN             : 'Host Last Seen',
        RNA_HOST_ROUTER_BRIDGE         : 'Host Identified as Router or Bridge',
        RNA_FLOW_STATISTICS            : 'Flow Statistics',
        RNA_VLAN_TAG_UPDATE            : 'VLAN Tag Information Update',
        RNA_HOST_DELETED_LIMIT         : 'Host Deleted: Limit Reached',
        RNA_CLIENT_APP_TIMEOUT         : 'Client Application Timeout',
        RNA_NETBIOS_NAME               : 'NetBIOS Name Change',
        RNA_NETBIOS_DOMAIN             : 'NetBIOS Domain Change',
        RNA_HT_DROPPED                 : 'Host Dropped: Host Limit Reached',
        RNA_BANNER_UPDATE              : 'Banner Update',
        RNA_TCP_SERVICE_CONFIDENCE     : 'TCP Service Confidence Update',
        RNA_UDP_SERVICE_CONFIDENCE     : 'UDP Service Confidence Update',
        RNA_OS_CONFIDENCE              : 'OS Confidence Update',
        RNA_DHCP_INFO                  : 'DHCP Information',
        RNA_IDENTITY_CONFLICT          : 'Identity Conflict',
        RNA_IDENTITY_TIMEOUT           : 'Identity Timeout',
        RNA_SECONDARY_UPDATE           : 'Secondary Host Update',
        RNA_CLIENT_APP_UPDATE          : 'Client Application Update',
    },
    RNA_TYPE_USER_INPUT : {
        RNA_SET_VALID_VULN             : 'Valid Vulnerabilities',
        RNA_SET_INVALID_VULN           : 'Invalid Vulnerabilities',
        RNA_DELETE_ADDRESS             : 'Delete Address',
        RNA_DELETE_SERVICE             : 'Delete Service',
        RNA_SET_HOST_CRITICALITY       : 'Set Host Criticality',
        RNA_ADD_HOST_ATTRIBUTE         : 'Host Attribute Add',
        RNA_UPDATE_HOST_ATTRIBUTE      : 'Host Attribute Update',
        RNA_DELETE_HOST_ATTRIBUTE      : 'Host Attribute Delete',
        RNA_SET_HOST_ATTRIBUTE         : 'Host Attribute Set Value',
        RNA_CLEAR_HOST_ATTRIBUTE       : 'Host Attribute Delete Value',
        RNA_ADD_SCAN_RESULTS           : 'Add Scan Result',
        RNA_USER_VULN_QUALIFY          : 'Set Vulnerability Impact Qualification',
        RNA_USER_POLICY_CONTROL        : 'User Policy Control',
        RNA_DELETE_PROTOCOL            : 'Delete Protocol',
        RNA_DELETE_CLIENT_APP          : 'Delete Client Application',
        RNA_SET_OS                     : 'Set Operating System Definition',
        RNA_ACCOUNT_SEEN               : 'Account Seen',
        RNA_ACCOUNT_UPDATE             : 'Account Update',
        RNA_SET_SERVICE                : 'Set Service Definition',
        RNA_DELETE_ADDR_V2             : 'Delete Host/Network',
        RNA_DELETE_SERVICE_V2          : 'Delete Service',
        RNA_VULN_VALID_V2              : 'Vulnerability Set Valid',
        RNA_VULN_INVALID_V2            : 'Vulnerability Set Invalid',
        RNA_CRITICALITY_V2             : 'Set Host Criticality',
        RNA_SET_ATTRIBUTE_VALUE_V2     : 'Host Attribute Set Value',
        RNA_DELETE_ATTRIBUTE_VALUE_V2  : 'Host Attribute Delete Value',
        RNA_ADD_HOST                   : 'Add Host',
        RNA_ADD_SERVICE                : 'Add Service',
        RNA_ADD_CLIENT_APP             : 'Add Client Application',
        RNA_ADD_PROTOCOL               : 'Add Protocol',
        RNA_RELOAD_APP_FPS             : 'Reload App',
        RNA_ACCOUNT_DELETE             : 'Account Delete',
        RNA_IOC_DELETE                 : 'IOC Delete',
        RNA_IOC_INVALUD                : 'IOC Invalid',
    },
    RNA_TYPE_FLOW : {
        RNA_FLOW_STATS                 : 'Flow Statistics',
        RNA_FLOW_CHUNK                 : 'Flow Chunk',
    },
    RUA_TYPE : {
        RUA_USER_INFO                  : 'User Information',
        RUA_LOGIN_INFO                 : 'User Login Information',
        RUA_REMOVE_USER                : 'Delete User Identity',
        RUA_DROP_USER                  : 'User Identity Dropped: User Limit Reached',
    },
    IOC_TYPE : {
        IOC_SET                        : 'IOC Set',
    },
}

TYPES = {
    # 'METADATA' numbers are metadata record types that eNcore will decode and cache.
    'METADATA': [
        4, 54, 55, 59, 60, 62, 63, 66, 67, 69, 70, 90, 91, 96, 98, 109,
        111, 115, 116, 117, 118, 119, 120, 121, 122, 123, 124, 127, 128,
        129, 130, 161, 260, 270, 280, 281, 320, 321, 510, 511, 515, 516,
        517, 518, 519, 520, 530, 602, 604, 605, 606, 608, 700 ],

    'PACKET': [ 2 ],

    'INTRUSION': [ 104, 105, 207, 208, 400 ],

    'FILE_MALWARE': [ 125, 500, 502 ],

    'CORRELATION': [ 69, 112 ],

    'CONNECTION': [ 71 ],

    'RNA': [
        10, 11, 12, 13, 14, 15, 16, 17, 19, 20, 21, 22, 25, 26, 27,
        28, 31, 35, 101, 103, 107, 160 ],

    'RUA': [ 92, 93, 94, 95 ],

    'EVENT': [
        9, 110, 112, 131, 145, 146, 322]
}
